(*
   APFELM V1.0

   Draw an "Apfelmaennchen" (the Mandelbrot set).
*)
program Plot3D;
(*$IKERNEL2.INC*)
(*$IINPUT.INC*)

const
  MaxItStandard = 100; (* Maximum number of iterations for the standard graphic *)
  ScreenWidth   = 720;
  ScreenHeight  = 256;
  MaxX          = 719;
  MaxY          = 255;

var
  maxIt:    integer; (* Maximum number of iterations *)
  standard: boolean; (* Draw standard Mandelbrot set? *)
  preview:  boolean; (* Draw in fast preview mode? *)


(* Iterates the Mandelbrot function using optimizations *)
(* for the standard Mandelbrot set.                     *)
function iterOptimized(cx, cy: real): integer;
var cnt: integer;
    zx, zy: real;
    zxq, zyq: real;
    cxq, cyq: real;
    r, s: real;
begin
    cnt := 0;
    zx := 0; zy := 0;
    zxq := 0; zyq := 0;
    cxq := cx*cx; cyq := cy*cy;
    r := cxq + cyq;
    s := SQRT(r - 0.5*cx + 0.0625);
    if (16*r*s > 5*s - 4*cx + 1) and ((cx+1)*(cx+1) + cyq > 0.0625) then begin
        repeat
            zy := 2*zx*zy + cy;
            zx := zxq - zyq + cx;
            zxq := zx*zx;
            zyq := zy*zy;
            cnt := succ(cnt);
        until (zxq + zyq >= 4) or (cnt >= maxIt)
    end
    else begin
        cnt := maxIt;
    end;
    
    iterOptimized := cnt;
end; (* iterOptimized *)


(* Iterates the Mandelbrot function - no optimizations. *)
function iter(cx, cy: real): integer;
var cnt: integer;
    zx, zy: real;
    zxq, zyq: real;
begin
    cnt := 0;
    zx := 0; zy := 0;
    zxq := 0; zyq := 0;
    repeat
        zy := 2*zx*zy + cy;
        zx := zxq - zyq + cx;
        zxq := zx*zx;
        zyq := zy*zy;
        cnt := succ(cnt);
    until (zxq + zyq >= 4) or (cnt >= maxIt);

    iter := cnt;
end; (* iter *)


(* Draws a dot. The size of the dot depends on the mode: *)
(* in preview mode the dot is 4x4 pixels large, in       *)
(* normal mode its size is a single pixel.               *)
procedure drawDot(x, y: integer);
var xt, yt: integer;
begin
    y := MaxY - y; (* move origin from top left to bottom left *)
    if preview then begin (* plot 4x4 pixels in total in preview mode *)
        for xt := x to x + 3 do begin
            yt := y;
            Plot(xt, yt, 0);
            yt := succ(yt);
            Plot(xt, yt, 0);
            yt := succ(yt);
            Plot(xt, yt, 0);
            yt := succ(yt);
            Plot(xt, yt, 0);
        end;
    end
    else begin (* plot a single pixel in non-preview mode *)
        Plot(x, y, 0);
    end;
end; (* drawDot *)


(* Calculates a dot. *)
procedure calcDot(cx, cy: real; x, y: integer);
var cnt: integer;
begin
    if standard then
        cnt := iterOptimized(cx, cy)
    else
        cnt := iter(cx, cy);

    if cnt mod 2 = 1 then begin
        drawDot(x, y);
        if standard then begin (* plot the mirror dot *)
            drawDot(x, ScreenHeight - y);
        end;
    end;
end; (* calcDot *)


(* Draws the mandelbrot set.        *)
(* Parameters: X and Y value ranges *)
procedure draw(xa, xe, ya, ye: real);
var y0Pkt: integer;
    cy0Pkt: real;
    dx, dy: real;
    dx4, dy4: real;
    cx, cy: real;
    x, y: integer;
begin
    HideCursor;
    ClrScr;
    
    dx := (xe-xa) / ScreenWidth;
    dy := (ye-ya) / ScreenHeight;
    
    if standard then begin
        cy0Pkt := 0;
        y0Pkt := trunc(ye / (ye-ya) * ScreenHeight);
        dy := -dy;
    end else begin
        cy0Pkt := ya;
        y0Pkt := 0;
    end;

    dx4 := 4 * dx;
    dy4 := 4 * dy;
    
    cx := xa;
    x := 0;
    while x < ScreenWidth do begin
        cy := cy0Pkt;
        y := y0Pkt;
        while y < ScreenHeight do begin
        
            calcDot(cx, cy, x, y);
            
            if preview then begin
                y := y + 4; (* = step 4 *)
                cy := cy + dy4;
            end
            else begin
                y := succ(y);
                cy := cy + dy;
            end;
        end;
        if preview then begin
            x := x + 4; (* = step 4 *)
            cx := cx + dx4;
        end
        else begin
            x := succ(x);
            cx := cx + dx;
        end;
    end;
    
    Beep;
    WaitForKey;
end; (* draw *)


(* Lets the user enter the x and y range, then draws the picture. *)
procedure inputCustomValues(row: integer);
var tmp: real;
    xa, xe, ya, ye: real;
begin
  GotoXY(28, row); Write('X von: '); xa := InputReal(15, ','); row := succ(row);
  GotoXY(28, row); Write('X bis: '); xe := InputReal(15, ','); row := succ(row);
  GotoXY(28, row); Write('Y von: '); ya := InputReal(15, ','); row := succ(row);
  GotoXY(28, row); Write('Y bis: '); ye := InputReal(15, ','); row := succ(row);
  GotoXY(28, row); Write('Max. Iterationstiefe: '); maxIt := InputNaturalNumber(4);
  
  if xa > xe then begin
    tmp := xa;
    xa := xe;
    xe := tmp;
  end;
  
  if ya > ya then begin
    tmp := ya;
    ya := ye;
    ye := tmp;
  end;
  
  draw(xa, xe, ya, ye);
end; (* inputCustomValues *)


(* Displays the mode (normal or preview) *)
procedure displayMode(row: integer);
begin
  GotoXY(45, row);
  if preview = true then
    Write('Vorschau (niedrige Aufl|sung)')
  else
    Write('Normal (hohe Aufl|sung)      ');
end; (* displayMode *)


(* Displays the main menu and branches according to the selection. *)
procedure mainMenu;
var selection: char;
    stop: boolean;
    row: integer;
begin
  row := 4;         GotoXY(36,row); Write('Apfelm{nnchen V1.0');
  row := succ(row); GotoXY(38,row); Write('von Bernd Bock');
  row := 10;        GotoXY(28,row); Write('1: Standard-Apfelm{nnchen');
  row := succ(row); GotoXY(28,row); Write('2: Benutzerdefinierter Ausschnitt');
  row := succ(row); GotoXY(28,row); Write('3: Vorschau-Modus umschalten');
  row := succ(row); GotoXY(28,row); Write('0: Ende');
  row := row + 2;   GotoXY(28,row); Write('Aktueller Modus:'); displayMode(row);
  row := row + 2;   GotoXY(28,row); Write('Auswahl:');

  repeat
    stop := false;
    GotoXY(37,row);
    selection := InputChar;
    case selection of
      '1': begin
             standard := true;
             maxIt := MaxItStandard;
             draw(-2.05, 0.75, -0.9625, 0.9625); (* draws standard Mandelbrot set *)
           end;
      '2': begin
             standard := false;
             inputCustomValues(row + 5); (* inputs custom values, then draws *)
           end;
      '3': begin
             preview := not preview;
             displayMode(row - 2);
           end;
      '0': stop := true;
    end;
  until stop;
end; (* mainMenu *)

(* Main program *)
begin
  Write(chr(27), '0'); (* switch off status bar *)
  GraphInit;
  ClrScr;

  standard := true;
  preview  := true;
  
  mainMenu;
  
  Beep;
  
  Write(chr(27), '1'); (* switch on status bar *)
  ShowCursor;
end.
